@@ -8,9 +8,7 @@
 
 /* derived from LINUX, VOXWARE and SUN for MiNT Audio Device by Petr Stehlik */
 #include <fcntl.h>
-#include "config.h"
 #include "mpg123.h"
-#include "debug.h"
 #include <ioctl.h>
 #include <audios.h>
 
