@@ -6,18 +6,11 @@
 	initially written by Martin Denn
 */
 
-#include <sys/types.h>
-#include <stdio.h>
-#include <unistd.h>
+#include "mpg123.h"
 #include <fcntl.h>
-#include <stdlib.h>
 #include <audio/audiolib.h>
 #include <audio/soundlib.h>
 
-#include "config.h"
-#include "mpg123.h"
-#include "debug.h"
-
 typedef struct
 {
     AuServer            *aud;
