@@ -6,16 +6,10 @@
 	initially written by Michael Hipp
 */
 
-#include <sys/types.h>
 #include <sys/ioctl.h>
-#include <stdio.h>
-#include <unistd.h>
 #include <fcntl.h>
-#include <stdlib.h>
 
-#include "config.h"
 #include "mpg123.h"
-#include "debug.h"
 
 #ifdef HAVE_LINUX_SOUNDCARD_H
 #include <linux/soundcard.h>
