@@ -6,26 +6,17 @@
 	initially written by Michael Hipp
 */
 
-#include <ctype.h>
-#include <stdlib.h>
-#include <signal.h>
+#include "mpg123.h"
 
-#include <sys/types.h>
+/* #include <ctype.h> */
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <math.h>
-#ifdef HAVE_INTTYPES_H
-#include <inttypes.h>
-#endif
 
 #include <fcntl.h>
 
-#include "config.h"
-#include "mpg123.h"
 #include "id3.h"
 #include "icy.h"
 #include "common.h"
-#include "debug.h"
 
 #ifdef WIN32
 #include <winsock.h>
@@ -63,13 +54,6 @@ int abr_rate = 0;
 #endif
 unsigned long track_frames = 0;
 /* a limit for number of frames in a track; beyond that unsigned long may not be enough to hold byte addresses */
-#ifdef HAVE_LIMITS_H
-#include <limits.h>
-#endif
-#ifndef ULONG_MAX
-/* hm, is this portable across preprocessors? */
-#define ULONG_MAX ((unsigned long)-1)
-#endif
 #define TRACK_MAX_FRAMES ULONG_MAX/4/1152
 
 /* this could become a struct... */
@@ -139,6 +123,7 @@ unsigned long samples_to_bytes(unsigned long s, struct frame *fr , struct audio_
 
 void audio_flush(int outmode, struct audio_info_struct *ai)
 {
+	/* the gapless code is not in effect for buffered mode... as then condition for audio_flush is never met */
 	#ifdef GAPLESS
 	if(param.gapless) layer3_gapless_buffercheck();
 	#endif
@@ -153,6 +138,7 @@ void audio_flush(int outmode, struct audio_info_struct *ai)
 				audio_play_samples (ai, pcm_sample, pcm_point);
 			break;
 			case DECODE_BUFFER:
+				error("The buffer doesn't work like that... I shouldn't ever be getting here.");
 				write (buffer_fd[1], pcm_sample, pcm_point);
 			break;
 			case DECODE_WAV:
@@ -187,7 +173,7 @@ void (*catchsignal(int signum, void(*handler)()))()
 
 void read_frame_init (struct frame* fr)
 {
-	fr->num = 0;
+	fr->num = -1;
 	oldhead = 0;
 	firsthead = 0;
 	vbr = CBR;
@@ -785,6 +771,7 @@ init_resync:
 	{
 		mean_framesize = ((mean_frames-1)*mean_framesize+compute_bpf(fr)) / mean_frames ;
 	}
+	++fr->num; /* 0 for the first! */
 	/* index the position */
 	if(INDEX_SIZE > 0) /* any sane compiler should make a no-brainer out of this */
 	{
@@ -808,7 +795,6 @@ init_resync:
 			}
 		}
 	}
-	++fr->num;
   return 1;
 }
 
@@ -995,11 +981,10 @@ debug2("bitrate index: %i (%i)", fr->bitrate_index, tabsel_123[fr->lsf][1][fr->b
 
 /* concurring to print_rheader... here for control_generic */
 const char* remote_header_help = "S <mpeg-version> <layer> <sampling freq> <mode(stereo/mono/...)> <mode_ext> <framesize> <stereo> <copyright> <error_protected> <emphasis> <bitrate> <extension> <vbr(0/1=yes/no)>";
-void make_remote_header(struct frame* fr, char *target)
+void print_remote_header(struct frame* fr)
 {
-	/* redundancy */
 	static char *modes[4] = {"Stereo", "Joint-Stereo", "Dual-Channel", "Single-Channel"};
-	snprintf(target, 1000, "S %s %d %ld %s %d %d %d %d %d %d %d %d %d",
+	generic_sendmsg("S %s %d %ld %s %d %d %d %d %d %d %d %d %d",
 		fr->mpeg25 ? "2.5" : (fr->lsf ? "2.0" : "1.0"),
 		fr->lay,
 		freqs[fr->sampling_frequency],
@@ -1015,24 +1000,6 @@ void make_remote_header(struct frame* fr, char *target)
 		vbr);
 }
 
-
-#ifdef MPG123_REMOTE
-void print_rheader(struct frame *fr)
-{
-	static char *modes[4] = { "Stereo", "Joint-Stereo", "Dual-Channel", "Single-Channel" };
-	static char *layers[4] = { "Unknown" , "I", "II", "III" };
-	static char *mpeg_type[2] = { "1.0" , "2.0" };
-
-	/* version, layer, freq, mode, channels, bitrate, BPF, VBR*/
-	fprintf(stderr,"@I %s %s %ld %s %d %d %d %i\n",
-			mpeg_type[fr->lsf],layers[fr->lay],freqs[fr->sampling_frequency],
-			modes[fr->mode],fr->stereo,
-			vbr == ABR ? abr_rate : tabsel_123[fr->lsf][fr->lay-1][fr->bitrate_index],
-			fr->framesize+4,
-			vbr);
-}
-#endif
-
 void print_header(struct frame *fr)
 {
 	static char *modes[4] = { "Stereo", "Joint-Stereo", "Dual-Channel", "Single-Channel" };
