@@ -1,15 +1,15 @@
 /*
 	stringbuf: mimicking a bit of C++ to more safely handle strings
 
-	copyright 2006 by the mpg123 project - free software under the terms of the LGPL 2.1
-	see COPYING and AUTHORS files in distribution or http://mpg123.de
+	copyright 2006-7 by the mpg123 project - free software under the terms of the LGPL 2.1
+	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Thomas Orgis
 */
 
 #include "config.h"
 #include "debug.h"
 #include "stringbuf.h"
-#include <stdlib.h>
+#include "mpg123.h" /* actually just for safe_realloc */
 
 void init_stringbuf(struct stringbuf* sb)
 {
@@ -29,9 +29,13 @@ void free_stringbuf(struct stringbuf* sb)
 
 int resize_stringbuf(struct stringbuf* sb, size_t new)
 {
+	debug3("resizing string pointer %p from %lu to %lu", (void*) sb->p, (unsigned long)sb->size, (unsigned long)new);
 	if(sb->size != new)
 	{
-		char* t = (char*) realloc(sb->p, new*sizeof(char));
+		char* t;
+		debug("really!");
+		t = (char*) safe_realloc(sb->p, new*sizeof(char));
+		debug1("realloc returned %p", (void*) t); 
 		if(t != NULL)
 		{
 			sb->p = t;
@@ -78,3 +82,9 @@ int add_to_stringbuf(struct stringbuf* sb, char* stuff)
 	}
 	return 1;
 }
+
+int set_stringbuf(struct stringbuf* sb, char* stuff)
+{
+	sb->fill = 0;
+	return add_to_stringbuf(sb, stuff);
+}
