@@ -9,16 +9,12 @@
 	See xfermem.h for documentation/description.
 */
 
-#include "config.h"
 
 #ifndef NOXFERMEM
 
-#include <stdio.h>
-#include <stdlib.h>
+#include "mpg123.h"
 #include <string.h>
-#include <unistd.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/time.h>
 #include <sys/uio.h>
 #include <sys/mman.h>
@@ -29,7 +25,6 @@
 #include <sys/select.h>
 #endif
 
-#include "mpg123.h"
 
 #ifndef HAVE_MMAP
 #include <sys/ipc.h>
